import { world, EntityEquippableComponent, EquipmentSlot, ItemLockMode, ItemDurabilityComponent, ItemEnchantableComponent, ItemStack, GameMode } from '@minecraft/server';
import pickaxe_level from './pickaxe_level';
import ore_map from './ore_map';

function getRadiusRange(location, radius = 1) {
    const centerX = location.x;
    const centerY = location.y;
    const centerZ = location.z;
    const positions = [];
    for (let x = centerX - radius; x <= centerX + radius; x++) {
        for (let y = centerY - radius; y <= centerY + radius; y++) {
            for (let z = centerZ - radius; z <= centerZ + radius; z++) {
                positions.push({ x, y, z });
            }
        }
    }
    return positions;
}

function getRandomRangeValue(min, max) {
    min = Math.ceil(min);
    max = Math.floor(max);
    return Math.floor(Math.random() * (max - min + 1)) + min;
}

function splitGroups(sum, groupSize = 64) {
    const groups = [];
    while (sum > 0) {
        const group = Math.min(sum, groupSize);
        groups.push(group);
        sum -= group;
    }
    return groups;
}

const isSurvivalPlayer = (dimension, player) => dimension.getPlayers({ gameMode: GameMode.survival }).some((p) => p.name === player.name);
world.afterEvents.playerBreakBlock.subscribe(async (e) => {
    const { dimension, player, block } = e;
    const currentBreakBlock = e.brokenBlockPermutation;
    const blockTypeId = currentBreakBlock.type.id;
    digOre(player, dimension, block.location, blockTypeId);
});
async function digOre(player, dimension, location, blockTypeId) {
    const equipmentInventory = player.getComponent(EntityEquippableComponent.componentId);
    if (!equipmentInventory)
        return;
    const mainHand = equipmentInventory.getEquipmentSlot(EquipmentSlot.Mainhand);
    try {
        const currentSlotItem = mainHand.getItem();
        if (!currentSlotItem)
            return;
        const pickaxe = pickaxe_level[currentSlotItem.typeId];
        if (!player.isSneaking || !currentSlotItem.hasTag('is_pickaxe'))
            return;
        const survivalPlayer = isSurvivalPlayer(dimension, player);
        if (survivalPlayer)
            mainHand.lockMode = ItemLockMode.slot;
        const itemDurability = currentSlotItem.getComponent(ItemDurabilityComponent.componentId);
        const enchantments = currentSlotItem.getComponent(ItemEnchantableComponent.componentId);
        if (!enchantments || !itemDurability)
            return;
        const unbreaking = enchantments.getEnchantment('unbreaking')?.level || 0;
        const silk_touch = enchantments.hasEnchantment('silk_touch');
        const fortune = enchantments.getEnchantment('fortune')?.level || 0;
        let itemMaxDamage = itemDurability.damage * (1 + unbreaking);
        const itemMaxDurability = itemDurability.maxDurability * (1 + unbreaking);
        const blockTypeIdRemoveLit = blockTypeId.replace('lit_', '');
        const set = new Set();
        const stack = [...getRadiusRange(location)];
        while (stack.length > 0) {
            const _block = dimension.getBlock(stack.shift());
            if (!_block)
                continue;
            const typeId = _block.typeId;
            const isEqual = typeId.replace('lit_', '') === blockTypeIdRemoveLit;
            if (isEqual && pickaxe.includes(typeId)) {
                const pos = JSON.stringify(_block.location);
                if (set.has(pos))
                    continue;
                itemMaxDamage++;
                if (survivalPlayer && itemMaxDamage >= itemMaxDurability) {
                    continue;
                }
                await new Promise((resolve) => {
                    _block.setType('air');
                    resolve();
                });
                set.add(pos);
                stack.push(...getRadiusRange(_block.location));
            }
        }
        const _ore = ore_map[blockTypeId];
        if (silk_touch && _ore.support.silk_touch) {
            splitGroups(set.size).forEach((group) => {
                dimension.spawnItem(new ItemStack(blockTypeIdRemoveLit, group), location);
            });
        }
        else {
            const ore = {
                item: _ore.item,
                xp: [..._ore.xp],
                probability: [..._ore.probability]
            };
            if (fortune && _ore.support.fortune) {
                const maxProbability = ore.probability.pop();
                ore.probability.push(maxProbability + fortune);
            }
            const oreMap = { item: 0, xp: 0 };
            for (let i = 0; i < set.size; i++) {
                oreMap.xp += getRandomRangeValue(ore.xp[0], ore.xp[1]);
                oreMap.item += getRandomRangeValue(ore.probability[0], ore.probability[1]);
            }
            for (let i = 0; i < oreMap.xp; i++) {
                dimension.spawnEntity('xp_orb', player.location);
            }
            splitGroups(oreMap.item).forEach((group) => {
                dimension.spawnItem(new ItemStack(ore.item, group), location);
            });
        }
        if (survivalPlayer) {
            const damage = Math.ceil((itemMaxDamage * 1) / (1 + unbreaking));
            itemDurability.damage = damage > itemDurability.maxDurability ? itemDurability.maxDurability : damage;
            mainHand.setItem(currentSlotItem);
        }
    }
    catch (_error) {
        const error = _error;
        console.error(error.name);
        console.error(error.message);
        console.error(error);
    }
    finally {
        mainHand.lockMode = ItemLockMode.none;
    }
}